@@ -25,7 +25,10 @@ public:
   virtual void getShapeName(char* buffer, int buflen) { strncpy(buffer, "text", buflen); };
 
   int getElementCount(void){ return textArray.size(); }
-  
+  int getAttributeCount(AABox& bbox, AttribID attrib);
+  void getAttribute(AABox& bbox, AttribID attrib, int first, int count, double* result);
+  String getTextAttribute(AABox& bbox, AttribID attrib, int index);
+    
   Vertex getElementCenter(int index) { return vertexArray[index]; }
 
   void drawBegin(RenderContext* renderContext);
